<?php

include "_config/display.php";


class PDF_DashedRect extends FPDF
{
	function SetDash($black=null, $white=null)
    {
        if($black!==null)
            $s=sprintf('[%.3F %.3F] 0 d',$black*$this->k,$white*$this->k);
        else
            $s='[] 0 d';
        $this->_out($s);
    }
	
	function Polygon($points, $style='D')
	{
	    //Draw a polygon
	    if($style=='F')
	        $op = 'f';
	    elseif($style=='FD' || $style=='DF')
	        $op = 'b';
	    else
	        $op = 's';
	
	    $h = $this->h;
	    $k = $this->k;
	
	    $points_string = '';
	    for($i=0; $i<count($points); $i+=2){
	        $points_string .= sprintf('%.2F %.2F', $points[$i]*$k, ($h-$points[$i+1])*$k);
	        if($i==0)
	            $points_string .= ' m ';
	        else
	            $points_string .= ' l ';
	    }
	    $this->_out($points_string . $op);
	}

    function DashedRect($x1, $y1, $x2, $y2, $epaisseur=1, $nbPointilles=15)
    {
        $this->SetLineWidth($epaisseur);
        $longueur=abs($x1-$x2);
        $hauteur=abs($y1-$y2);
        if($longueur>$hauteur) {
            $Pointilles=($longueur/$nbPointilles)/2; // taille des pointilles
        }
        else {
            $Pointilles=($hauteur/$nbPointilles)/2;
        }
        for($i=$x1;$i<=$x2;$i+=$Pointilles+$Pointilles) {
            for($j=$i;$j<=($i+$Pointilles);$j++) {
                if($j<=($x2-1)) {
                    $this->Line($j,$y1,$j+1,$y1); // on trace le pointillé du haut, point par point
                    $this->Line($j,$y2,$j+1,$y2); // on trace le pointillé du bas, point par point
                }
            }
        }
        for($i=$y1;$i<=$y2;$i+=$Pointilles+$Pointilles) {
            for($j=$i;$j<=($i+$Pointilles);$j++) {
                if($j<=($y2-1)) {
                    $this->Line($x1,$j,$x1,$j+1); // on trace le pointillé du haut, point par point
                    $this->Line($x2,$j,$x2,$j+1); // on trace le pointillé du bas, point par point
                }
            }
        }
    }
	
	function Circle($x, $y, $r, $style='D')
	{
		$this->Ellipse($x,$y,$r,$r,$style);
	}

	function Ellipse($x, $y, $rx, $ry, $style='D')
	{
		if($style=='F')
			$op='f';
		elseif($style=='FD' || $style=='DF')
			$op='B';
		else
			$op='S';
		$lx=4/3*(M_SQRT2-1)*$rx;
		$ly=4/3*(M_SQRT2-1)*$ry;
		$k=$this->k;
		$h=$this->h;
		$this->_out(sprintf('%.2F %.2F m %.2F %.2F %.2F %.2F %.2F %.2F c',
			($x+$rx)*$k,($h-$y)*$k,
			($x+$rx)*$k,($h-($y-$ly))*$k,
			($x+$lx)*$k,($h-($y-$ry))*$k,
			$x*$k,($h-($y-$ry))*$k));
		$this->_out(sprintf('%.2F %.2F %.2F %.2F %.2F %.2F c',
			($x-$lx)*$k,($h-($y-$ry))*$k,
			($x-$rx)*$k,($h-($y-$ly))*$k,
			($x-$rx)*$k,($h-$y)*$k));
		$this->_out(sprintf('%.2F %.2F %.2F %.2F %.2F %.2F c',
			($x-$rx)*$k,($h-($y+$ly))*$k,
			($x-$lx)*$k,($h-($y+$ry))*$k,
			$x*$k,($h-($y+$ry))*$k));
		$this->_out(sprintf('%.2F %.2F %.2F %.2F %.2F %.2F c %s',
			($x+$lx)*$k,($h-($y+$ry))*$k,
			($x+$rx)*$k,($h-($y+$ly))*$k,
			($x+$rx)*$k,($h-$y)*$k,
			$op));
	}
}

if(isset($_GET['cube']))
{	

	$pdf = new PDF_DashedRect('L');
	$pdf->SetAutoPageBreak(false,0);

	for($m=0;$m<20;$m++)
	{ 

		$pdf->SetDrawColor(0);
		$pdf->SetTextColor(0);
	
		$c='';
		$sol=$c;
		
		$niveau=40;
		if($_GET['niveau']=='1') $niveau=55;
		if($_GET['niveau']=='2') $niveau=60;
		if($_GET['niveau']=='3') $niveau=75;
		if($_GET['niveau']=='4') $niveau=90;
		
		//BLancs
		for($i=0;$i<$niveau;$i++)
		{
			$rand=rand(0,96);
			$c=substr($c,0,$rand).' '.substr($c,$rand+1);
		}
		
		
		
		$pdf->AddPage('L');
		$pdf->SetFont('Arial','B',12);
		
		//Cadre pointillé
		$pdf->SetLineWidth(0.5);
		$pdf->DashedRect(30, 15, 270, 195, 0.5);
		
		$pdf->SetLineWidth(1);
		DrawSquare($pdf, 90, 15);
		DrawSquare($pdf, 30, 75);
		DrawSquare($pdf, 90, 75);
		DrawSquare($pdf, 210, 75);
		DrawSquare($pdf, 150, 75);
		DrawSquare($pdf, 90, 135);
		
		//Diagonales
		$pdf->SetLineWidth(0.25);
		$pdf->SetDrawColor(195);
		
		$pdf->Line(30,15,90,75);	
		$pdf->Line(150,75,210,15);
		$pdf->Line(210,75, 210, 15);
		$pdf->Line(210,75, 270, 15);
		
		$pdf->Line(90,135, 30, 195);
		$pdf->Line(150,135, 210,195);
		$pdf->Line(210,135, 210, 195);
		$pdf->Line(210,135, 270, 195);
		
		$pdf->SetDrawColor(0);
		
		//Lettres en chiffres
		$n='ABCDEFGHIJKLMNOP';
		$hex='0123456789ABCDEF';
		$t=null;
		
		for($i=0;$i<25;$i++)
		{
			$rand=rand(0,16);
			$n=substr($n,$rand,1).substr($n,0,$rand).substr($n,$rand+1);
		}
		
		for($i=0;$i<strlen($n);$i++)
		{
		
			$t[substr($n,$i,1)]=($i+1);
			if(isset($_GET['hexa'])) $t[substr($n,$i,1)]=substr($hex,$i,1);
		}
		
		//Affichage des lettres
		for($i=0;$i<96;$i++)
		{
			//Cas 1 : carré du haut
			if($i<16)
			{
					$pdf->SetXY(($i%4)*15+90, floor($i/4)*15+15);
			}
			elseif($i<80)
			{
				$pdf->SetXY(($i%16)*15+30, floor($i/16)*15+60);
			}
			else
			{
				$pdf->SetXY(($i%4)*15+90, floor($i/4)*15-165);
			}
			$pdf->SetFont('Arial','');
			$pdf->Cell(15,15, '',1,0,'C');
		}
		
		//Instructions
		$pdf->SetFont('Arial','',8);
		$pdf->SetXY(30,7);
		$pdf->Cell(100,10,utf8_decode('Coupez ou pliez vers l\'intérieur du cube les lignes en tirets. Pliez les lignes diagonales fines vers l\'intérieur et les lignes verticales fines vers l\'extérieur. Le cube tiendra de lui-même.'));
		
		if(!isset($_GET['nosoluce']))
		{
			//Solution ?
			$pdf->SetDrawColor(128);
			$pdf->SetTextColor(128);
			$pdf->SetLineWidth(0.5);
			DrawSquare($pdf, 195, 140, 15, 3.75);
			DrawSquare($pdf, 180, 155, 15, 3.75);
			DrawSquare($pdf, 195, 155, 15, 3.75);
			DrawSquare($pdf, 210, 155, 15, 3.75);
			DrawSquare($pdf, 225, 155, 15, 3.75);
			DrawSquare($pdf, 195, 170, 15, 3.75);
			
			//Affichage des lettres
			$pdf->SetFont('Arial','',7);
			$c=$sol;
			for($i=0;$i<96;$i++)
			{
				//Cas 1 : carré du haut
				if($i<16)
				{
					$pdf->SetXY(($i%4)*3.75+189.5, floor($i/4)*3.75+134.5);
				}
				elseif($i<80)
				{
					$pdf->SetXY(($i%16)*3.75+174.5, floor($i/16)*3.75+145.5);
				}
				else
				{
					$pdf->SetXY(($i%4)*3.75+189.5, floor($i/4)*3.75-120.5);
				}
				$pdf->Cell(15,15, '',0,0,'C');
			}
		}
	}
	
	$pdf->Output();
	
	die();
}

function DrawPyramid(& $pdf, $sens=0)
{
	$t=file('/homez.309/serviceswy/_svg/_files/pyramoku.txt');
	$c=$t[rand(0,count($t)-1)];
	$sol=$c;
	
	$niveau=12;
	if($_GET['niveau']=='1') $niveau=18;
	if($_GET['niveau']=='2') $niveau=24;
	if($_GET['niveau']=='3') $niveau=30;
	if($_GET['niveau']=='4') $niveau=36;
	
	//BLancs
	for($i=0;$i<$niveau;$i++)
	{
		$rand=rand(0,36);
		$c=substr($c,0,$rand).' '.substr($c,$rand+1);
	}
	
	
	
	
	$pdf->SetLineWidth(0.25);
	$pdf->SetFillColor(220);
	
	$ct=60.62;

	//Polygone
	/*$pdf->Polygon(array(297-2*$ct/3, (210*$sens)-(2*$sens-1)*0, 297-4*$ct/3, (210*$sens)-(2*$sens-1)*105*2/3, 297-8*$ct/3, (210*$sens)-(2*$sens-1)*105*2/3, 297-10*$ct/3, (210*$sens)-(2*$sens-1)*0, 297-8*$ct/3, (210*$sens)-(2*$sens-1)*0,297-7*$ct/3, (210*$sens)-(2*$sens-1)*105*1/3, 297-5*$ct/3, (210*$sens)-(2*$sens-1)*105*1/3,297-4*$ct/3, (210*$sens)-(2*$sens-1)*0), 'F');
*/
	
	//Traits
	$pdf->SetDrawColor(190);
	$pdf->SetLineWidth(2);
	
	$pdf->SetDash(5,5);
	$pdf->Line(297, (210*$sens)-(2*$sens-1)*9*210/18, 297-1.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-1.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-4.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-4.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-6*$ct/3, (210*$sens)-(2*$sens-1)*9*210/18);
	
	$pdf->Line(297-12*$ct/3, (210*$sens)-(2*$sens-1)*9*210/18, 297-13.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-13.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-16.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-16.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-18*$ct/3, (210*$sens)-(2*$sens-1)*9*210/18);
	
	
	$pdf->SetDash(10,10);
	$pdf->Line(297-6*$ct/3, (210*$sens)-(2*$sens-1)*9*210/18, 297-7.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-7.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-10.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-10.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-12*$ct/3, (210*$sens)-(2*$sens-1)*9*210/18);
	
	
	$pdf->SetDash(1,4);
	$pdf->Line(297-3*$ct/3, (210*$sens)-(2*$sens-1), 297-4.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-4.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-7.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-7.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-9*$ct/3, (210*$sens)-(2*$sens-1));
	
	$pdf->SetDash(7,2);
	$pdf->Line(297-9*$ct/3, (210*$sens)-(2*$sens-1), 297-10.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-10.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-13.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-13.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-15*$ct/3, (210*$sens)-(2*$sens-1));
	
	$pdf->Line(297+3*$ct/3, (210*$sens)-(2*$sens-1), 297+1.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297+1.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-1.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18);
	$pdf->Line(297-1.5*$ct/3, (210*$sens)-(2*$sens-1)*4.5*210/18, 297-3*$ct/3, (210*$sens)-(2*$sens-1));
	
	
	
	
	$pdf->SetDrawColor(0);
	
	$pdf->SetDash();
	
	$pdf->SetLineWidth(2);
	$pdf->Line(0, 105, 297, 105);
	
	$pdf->SetLineWidth(1);
	
	//Grands triangles
		$l=297;
	for($i=0;$i<5;$i++)
	{
		if($i % 2 ==0)
			$pdf->Line($l, (210*$sens)+0, $l-$ct, 105);
		else
			$pdf->Line($l, 105, $l-$ct, (210*$sens)+0);
		
		$l=$l-$ct;
	}
	
		
	// 3 lignes fines
	$pdf->SetLineWidth(0.25);

	for($i=1;$i<=3;$i++)
	{
		$pdf->Line(0, (105*$sens)+105*$i/3, 297, (105*$sens)+105*$i/3);
	}
	
	
	$l=297+2*$ct;	
	for($i=0;$i<30;$i++)
	{
		$pdf->Line($l, (210*$sens)+0, $l-$ct, 105);
		$pdf->Line($l, (210*$sens)+0, $l+$ct, 105);
		
		$l=$l-$ct*2/3;
	}
	
	
	$pdf->SetDrawColor(0);
	
	//Lettres en chiffres
	$n='ABCDEFGHI';
	$t=null;
	
	for($i=0;$i<25;$i++)
	{
		$rand=rand(0,9);
		$n=substr($n,$rand,1).substr($n,0,$rand).substr($n,$rand+1);
	}
	
	for($i=0;$i<strlen($n);$i++)
	{
		$t[substr($n,$i,1)]=($i+1);
	}
	
	for($j=0;$j<3;$j++)
	{
		for($i=0;$i<12;$i++)
		{
			$x=(45-2*$ct/3+$i*($ct/3)+$j*($ct/3));
			$y=(210*$sens)-(2*$sens-1)*(105-(28+$j*(105/3)))-($sens*20);
			
			$pdf->SetFont('Arial','B');
			if ($t[$c[$j*12+$i]]=='6' || $t[$c[$j*12+$i]]=='9') $pdf->SetFont('Arial','BU');
			
			$pdf->SetXY($x, $y);
			$pdf->Cell(20,20,$t[$c[$j*12+$i]],0,0,'C');
			
			$pdf->SetXY($x+4*$ct, $y);
			$pdf->Cell(20,20,$t[$c[$j*12+$i]],0,0,'C');
			
		}
	}
}


if(isset($_GET['pyramide']))
{	
	$pdf = new PDF_DashedRect('L');
	$pdf->SetAutoPageBreak(false,0);

	for($i=0;$i<20;$i++)
	{
		
		$pdf->AddPage('L');
		$pdf->SetFont('Arial','B',30 );

		DrawPyramid($pdf);
		DrawPyramid($pdf, 1);
	}
	
	$pdf->Output();
	
	die();
}

function DrawSquare(& $pdf, $x, $y, $w=60, $h=15)
{
	$pdf->SetLineWidth($w/60);
	$pdf->Rect($x, $y, $w,$w);
	
	$pdf->SetLineWidth($w/120);
	for($i=0;$i<4;$i++)
	{
		for($j=0;$j<4;$j++)
			$pdf->Rect($x+$i*$h,$y+$j*$h,$h,$h);
	}
}

//GET
if($_GET['num'])
{
	$t=file('/homez.309/serviceswy/_svg/_files/___sudoku/grilles_'.$_GET['num'].'.txt');
	if($_GET['grid'])
	{
		for($x=0;$x<count($t);$x++)
		{
			$c=$t[$x];
			?>
			<div style="float:left;width:200px;margin:3px;text-align:center">
			<table width="100%" style="border:3px solid black;border-collapse:collapse">
			<?php		
			for($i=0;$i<9;$i++)
			{
				echo '<tr>';
				for($j=0;$j<9;$j++)
				{
					echo '<td style="border:1px solid black;text-align:center;'.($j==2||$j==5?'border-right:3px solid black;':'').($i==2||$i==5?'border-bottom:3px solid black;':'').'">'.substr($c,9*$i+$j,1).'</td>';
				}
				echo '</tr>';
			}
			?>
			</table>
			</div>
			<?php
		}
		die();
	}
	
	echo '<div style="font-family:courier">';
	for($i=0;$i<count($t);$i++)
	{
		$bId=true;
		for($j=0;$j<96;$j++)
		{
			if($i>0)
			{
				if(substr($t[$i-1],$j,1)!==substr($t[$i],$j,1))
				{
					echo '<span style="color:red">'.substr($t[$i],$j,1).'</span>';
					$bId=false;
				}
				else
				{
					echo substr($t[$i-1],$j,1);
				}
			}
			else
			{
				echo substr($t[$i],$j,1);
			}
		}
		if($bId && $i>0) echo ' IDENTIQUE !!!';
		echo '<br/>';
	}
	echo '</div>';

	die();
}

//Liste
$t=scandir('/homez.309/serviceswy/_svg/_files/___sudoku/');
$a=array();
for ($i=0;$i<count($t);$i++)
{
	$bg='';	
	if(substr($t[$i],0,8)=='grilles_')
	{
		$nb=substr($t[$i],8);
		$nb=substr($nb,0,strpos($nb,'.'));
		if($nb%2) $bg=' style="background-color:#ddd"';
		$min=(($nb-1)*10000)+1;
		$max=$nb*10000;
		$a[$nb]='<tr><td'.$bg.'>Combinaisons de '.number_format($min,0,',','.').' à '.number_format($max,0,',','.').'</td><td'.$bg.'><a target="_blank" href="?num='.$nb.'">Liste</a></td><td'.$bg.'><a target="_blank" href="?num='.$nb.'&grid=1">Grilles</a></td>';
	}
}

ksort($a);





if(isset($_GET['print']))
{
	
	$pdf = new FPDF();
	
	$pdf->AddPage();
	$pdf->SetFont('Arial','B',16);
	
	DrawGrille($pdf,0,0);
	DrawGrille($pdf,0,100);
	DrawGrille($pdf,100,0);
	DrawGrille($pdf,100,100);
	
	$pdf->Output();
	
	die();
}

function DrawGrille(& $pdf, $origX, $origY)
{
	//Chiffres
	$l="ABCDEFGHI";
	$n="123456789";

	for($i=0;$i<25;$i++)
	{
		$rand=rand(0,9);
		$n=substr($n,$rand,1).substr($n,0,$rand).substr($n,$rand+1);
	}
	
	$pdf->SetDrawColor(0);
	$pdf->SetLineWidth(1);
	$pdf->Rect($origX+10,$origY+10,90,90);
	
	
	for($i=0;$i<3;$i++)
	{
		for($j=0;$j<3;$j++)
			$pdf->Rect($origX+10+$i*30,$origY+10+$j*30,30,30);
	}
	
	
	$t=file('/homez.309/serviceswy/_svg/_files/___sudoku/grilles_'.rand(1,count($a)).'.txt');
	$c=$t[rand(0,count($t))];
	
	//Substitution des lettres en chiffres
	for($i=0;$i<9;$i++)
		$c=str_replace(substr($l,$i,1),substr($n,$i,1),$c);
	
	//BLancs
	for($i=0;$i<60;$i++)
	{
		$rand=rand(0,81);
		$c=substr($c,0,$rand).' '.substr($c,$rand+1);
	}
	
	//echo strlen($c);
	
	$pdf->SetLineWidth(0.5);
	for($i=0;$i<9;$i++)
	{
		for($j=0;$j<9;$j++)
		{
			$pdf->SetXY($origX+10+$j*10, $origY+10+$i*10);
			$pdf->Cell(10,10, substr($c,9*$i+$j,1),1,0,'C');
		}
	}
	
}

SetHead('Combinaisons de Sudoku classique, par fichiers de 10.000');

?>

		<h1>Combinaisons regroupées par 10.000</h1>
		<div style="float:right;width:200px;margin-left:20px;text-align:center;">
		<table width="100%" style="border:3px solid black;border-collapse:collapse">
		<?php
		$t=file('/homez.309/serviceswy/_svg/_files/___sudoku/grilles_'.rand(1,count($a)-1).'.txt');
		$c=$t[rand(0,count($t))];
		for($i=0;$i<9;$i++)
		{
			echo '<tr>';
			for($j=0;$j<9;$j++)
			{
				echo '<td style="border:1px solid black;text-align:center;'.($j==2||$j==5?'border-right:3px solid black;':'').($i==2||$i==5?'border-bottom:3px solid black;':'').'">'.substr($c,9*$i+$j,1).'</td>';
			}
			echo '</tr>';
		}
		?>
		</table>
		<i>Exemple de structure</i>
		</div>
		<p>
		Le but de cette page est de lister de manière exhaustive (si possible) toutes les structures de combinaisons possibles pour une grille de sudoku de 9x9</p>
		<p>
		La structure donne les emplacements possibles, notés de A à I, où A peut prendre les valeurs de 1 à 9, B de 1 à 9 mais différent de A, etc.
		</p>
		<p>
		Les combinaisons sont générées par une macro Excel, regroupées dans des fichiers TXT par lot de 10.000. Le nombre total de solutions, en incluant toutes les combinaisons numériques, sera donc de 9! (factorielle 9 = <b>362.280</b>) fois plus. </p><p>Théoriquement, je devrais obtenir près de <b>18 millions de milliards</b> de combinaisons, soit <b>1.800 milliards</b> de fichiers TXT, soit une base de <b>1,3 millions de To</b>. On verra bien !</p>
		<p>A quoi cela peut-il servir ?<ul>
		<li>Au côté technique et artistique</li>
		<li>A des personnes voulant faire des jeux de sudoku en ligne, avec déjà des grilles prédéfinies (<a target="_blank" href="?print">cliquez ici pour tester</a>)</li>
		<li>A augmenter ma collection de DVD une fois qu'ils seront gravés...</li>
		</ul>
		<p>
		<hr/>
		<table width="100%">
		<?php
		
		
		foreach($a as $nb=>$txt)
		{
			echo $a[$nb];
		}
		
		?>
		</table>
<?php

SetFoot();

?>